// standard includes
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <string.h>
#include <math.h>
#include <vector>

#include "query.h"

// Vector version of getvar:
double getvar(double energy, double pitch, vector<double> gap, 
	      vector<double> conv, double ssdthick, double incang, 
	      int numplanes);
//Partial Vector version
double getvar(double energy, double pitch, vector<double> gap,
	      double conv, double ssdthick, double incang,
	      int numplanes);
// Scalar version of getvar:
extern double getvar(double energy, double pitch, double gap,
		     double conv, double ssdthick, double incang, 
		     int numplanes);

main(int argc, char **argv) {
  
  double emin, emax, einc;
  double pitch=0.236;
  double conv;
  double ssd=0.01;
  double incang=0.0;
  int totalplanes=6;
  int minplanes=totalplanes-3;
  double avg=0.0;
  char config;
  
  ofstream outfile("avgout.psf");

  DblQuery mineQ("Minimum energy", "10"), maxeQ("Maximum energy", "10000");
  DblQuery numeQ("Number of energy steps", "50");

  Query fulleQ(mineQ, maxeQ, numeQ);
  fulleQ.textask();

  emin=mineQ.val(); emax=maxeQ.val(); einc=numeQ.val();
  outfile << einc << endl;
  einc=exp(log(emax/emin)/(einc-1));
  cout << "Multiplier for each step: " << einc << endl;
  for (double e=emin; e<=emax*1.1; e*=einc)
    outfile << log10(e) << endl;
  outfile << endl;
  outfile << "0" << endl;



  CharQuery configQ("Stretch (s), pancake (p), or neither (n)", "p");
  Query fullconfigQ(configQ);
  fullconfigQ.textask();
  config=configQ.val();

  vector<double> gap;
  gap.push_back(30.);
  gap[0]=30.0;

  if(config=='p')
    for(int i=totalplanes; i>0; i--)
      gap.push_back(30.0);

  else if(config=='s' || config=='S') {
    gap.push_back(30.0);
    for(int i=totalplanes-1; i>0; i--)
      gap.push_back(60.0);
  } else {
    DblQuery gapQ("Gap between planes", "30.0");
    Query fullgapQ(gapQ);
    fullgapQ.textask();
    for(int i=totalplanes; i>0; i--)
      gap.push_back(gapQ.val());
  }

  cout << "Gap: ";
  for(int i=0; i<=gap.size(); i++)
    cout << gap[i] << " ";
  cout << endl;
  
  DblQuery convQ("Radiator thickness", "0.035");
  Query fullconvQ(convQ);
  fullconvQ.textask();
  conv=convQ.val();

  DblQuery ssdQ("SSD thickness", "0.01");
  Query fullssdQ(ssdQ);
  fullssdQ.textask();
  ssd=ssdQ.val();

  double  normP= 1+exp(-7*(conv+ssd)/9.)+
                   exp(-14*(conv+ssd)/9.) +
                   exp(-21*(conv+ssd)/9.);

  for(double e=emin; e<=emax*1.1; e*=einc) {
    for(int numplanes=1; numplanes<=totalplanes-2; numplanes++) {
      double bob;
      double Pbob;

      bob=getvar(e/2, pitch, gap, conv, ssd, incang, totalplanes-numplanes+1);
      bob=bob/2.;  // that's for the 2 e-, in quadrature.
      Pbob=exp(-7*(numplanes-1)*(conv+ssd)/9.)/normP;
      avg+=Pbob*bob;
    }
    outfile << sqrt(avg)*57.29 << endl;
    avg=0;
  }
  cout << "Point-spread widths written to avgout.psf" << endl;
}

