// standard includes
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <string.h>
#include <math.h>

#include "query.h"
//#include "utils.h"

extern double getvar(double energy, double pitch, double gap,
		     double conv, double ssdthick, double incang, 
		     int numplanes);

main(int argc, char **argv) {

  double emin, emax, einc=1e4;
  double splitmin, splitmax, splitinc=1e4;
  double pitchmin, pitchmax, pitchinc=1e4;
  double gapmin, gapmax, gapinc=1e4;
  double convmin, convmax, convinc=1e4;
  double ssdthickmin, ssdthickmax, ssdthickinc=1e4;
  double incangmin, incangmax, incanginc=1e4;
  int numplanesmin, numplanesmax, numplanesinc=100;
  int eflag=0, splitflag=0, pitchflag=0, gapflag=0, convflag=0,
      ssdthickflag=0, incangflag=0;
  int nomoreflag=0; // set when we have two parameters already
  int horflag=0; // set when this variable will be the horizontal one.
  double psf;

  emin=10; emax=1000; einc=10;
  splitmin=0.25; splitmax=0.75; splitinc=3;
  pitchmin=0.236; pitchmax=0.236; pitchinc=1e4;
  gapmin=gapmax=30.0; convmin=convmax=0.035; ssdthickmin=ssdthickmax=0.01; 
  incangmin=incangmax=0; numplanesmin=numplanesmax=6;

  //  runNormal();
  ofstream outfile("out.psf");

  cout << endl;
  cout << "Enter parameters as requested.  To allow a parameter to vary, enter -1." << endl;
  cout << "Don't enter zero energy either as the total or the fraction." << endl;
  cout << "Also, don't enter a 'minimum' greater than the 'maximum'." << endl;


  //  cout << "Total gamma energy in MeV: " << flush;
  //  cin >> emin;
  DblQuery eQ("Total gamma energy in MeV", "100");
  Query initeQ(eQ);
  initeQ.textask();
  
  emin=eQ.val();
  emax=emin;
  if(emin<0) {
    DblQuery mineQ("Minimum energy", "10"), maxeQ("Maximum energy", "10000");
    DblQuery numeQ("Number of energy steps", "10");

    Query fulleQ(mineQ, maxeQ, numeQ);
    fulleQ.textask();

    emin=mineQ.val(); emax=maxeQ.val(); einc=numeQ.val();
    outfile << einc << endl;
    einc=exp(log(emax/emin)/(einc-1));
    cout << "Multiplier for each step: " << einc << endl;
    eflag=1; horflag=1;
    cout << "VERTICAL AXIS: Log of Total Energy" << endl;
    for (double e=emin; e<=emax*1.1; e*=einc)
      outfile << log10(e) << " ";
    outfile << endl;
  } 

  DblQuery splitQ("Fraction of energy in electron #1", "0.5");
  Query initsplitQ(splitQ);
  initsplitQ.textask();
  //  while(splitQ.val() <= 0 || splitQ.val >= 1)
  //    initsplitQ.textask();

  splitmin=splitQ.val();
  splitmax=splitmin;
  if(splitmin<0) {
    DblQuery splitminQ("Minimum fraction", "0.05"), 
      splitmaxQ("Maximum fraction", "0.5"),
      splitnumQ("Number of split fraction steps", "10");

    Query fullsplitQ(splitminQ, splitmaxQ, splitnumQ);
    fullsplitQ.textask();
    splitmin=splitminQ.val(); splitmax=splitmaxQ.val(); splitinc=splitnumQ.val();

    outfile << splitinc << endl;
    splitinc=(splitmax - splitmin)/(splitinc-1);
    cout << "Increment in split fraction: " << splitinc << endl;
    if(horflag) {
      cout << "HORIZONTAL AXIS: ";
      nomoreflag=1;  // now we have both axes
    } else {
      cout << "VERTICAL AXIS: ";
      horflag=1;
      splitflag=1;
    }
    cout << "Fraction of energy in electron #1" << endl;
    for(double split=splitmin; split<=splitmax+.3*splitinc; split+=splitinc) 
      outfile << split << " ";
    outfile << endl;
  }

  DblQuery pitchQ("Strip pitch in mm", "0.236");
  Query initpitchQ(pitchQ);
  initpitchQ.textask();
  pitchmin=pitchQ.val();
  pitchmax=pitchmin;
  if(pitchmin<0 && nomoreflag)
    while(pitchmin<0) {
      cout << "Already have two axes.  Enter single value for pitch: " << flush;
      cin >> pitchmin;
    }
  if(pitchmin<0) {
    DblQuery pitchminQ("Minimum pitch", "0.100"),
      pitchmaxQ("Maximum pitch", "0.500"),
      pitchnumQ("Number of pitch steps", "10");

    Query fullpitchQ(pitchminQ, pitchmaxQ, pitchnumQ);
    fullpitchQ.textask();
    pitchmin=pitchminQ.val(); pitchmax=pitchmaxQ.val(); pitchinc=pitchnumQ.val();

    outfile << pitchinc << endl;
    pitchinc=(pitchmax-pitchmin)/(pitchinc-1);
    if(horflag) {
      cout << "HORIZONTAL AXIS: ";
      nomoreflag=1;  // now we have both axes
    } else {
      cout << "VERTICAL AXIS: ";
      horflag=1;
      pitchflag=1;
    }
    cout << "Strip pitch" << endl;
    for(double pitch=pitchmin; pitch<=pitchmax+.3*pitchinc; pitch+=pitchinc) 
      outfile << pitch << " ";
    outfile << endl;

  }

  DblQuery gapQ("Gap between planes in mm", "30.0");
  Query initgapQ(gapQ);
  initgapQ.textask();
  gapmin=gapQ.val();
  gapmax=gapmin;
  if(gapmin<0 && nomoreflag)
    while(gapmin<0) {
      cout << "Already have two axes.  Enter single value for gap: " << flush;
      cin >> gapmin;
    }
  if(gapmin<0) {
    DblQuery gapminQ("Minimum gap", "10.0"),
      gapmaxQ("Maximum gap", "60.0"),
      gapnumQ("Number of gap steps", "10");

    Query fullgapQ(gapminQ, gapmaxQ, gapnumQ);
    fullgapQ.textask();
    gapmin=gapminQ.val(); gapmax=gapmaxQ.val(); gapinc=gapnumQ.val();

    outfile << gapinc << endl;
    gapinc=(gapmax - gapmin)/(gapinc - 1);
    if(horflag) {
      cout << "HORIZONTAL AXIS: ";
      nomoreflag=1;  // now we have both axes
    } else {
      cout << "VERTICAL AXIS: ";
      horflag=1;
      gapflag=1;
    }
    cout << "Gap" << endl;
    for(double gap=gapmin; gap<=gapmax+.3*gapinc; gap+=gapinc)
      outfile << gap << " ";
    outfile << endl;
  }

  DblQuery convQ("Converter thickness in rad. lengths", "0.035");
  Query initconvQ(convQ);
  initconvQ.textask();
  convmin=convQ.val();
  convmax=convmin;
  if(convmin<0 && nomoreflag)
    while(convmin<0) {
      cout << "Already have two axes.  Enter single value for converter thickness: " << flush;
      cin >> convmin;
    }
  if(convmin<0) {
    DblQuery convminQ("Minimum converter thickness", "0.0"),
      convmaxQ("Maximum converter thickness", "0.05"),
      convnumQ("Number of converter thickness steps", "20");
    
    Query fullconvQ(convminQ, convmaxQ, convnumQ);
    fullconvQ.textask();
    convmin=convminQ.val(); convmax=convmaxQ.val(); convinc=convnumQ.val();

    outfile << convinc << endl;
    convinc=(convmax-convmin)/(convinc-1);
    if(horflag) {
      cout << "HORIZONTAL AXIS: ";
      nomoreflag=1;  // now we have both axes
    } else {
      cout << "VERTICAL AXIS: ";
      horflag=1;
      convflag=1;
    }
    cout << "Converter thickness" << endl;
    for(double conv=convmin; conv<=convmax+.3*convinc; conv+=convinc)
      outfile << conv << " ";
    outfile << endl;
  }

  DblQuery ssdQ("SSD thickness per plane in rad. lengths", "0.01");
  Query initssdQ(ssdQ);
  initssdQ.textask();
  ssdthickmin=ssdQ.val();
  ssdthickmax=ssdthickmin;
  if(ssdthickmin<0 && nomoreflag)
    while(ssdthickmin<0) {
      cout << "Already have two axes.  Enter single value for SSD thickness: " << flush;
      cin >> ssdthickmin;
    }
  if(ssdthickmin<0) {
    DblQuery ssdminQ("Minimum SSD thickness", "0.005"),
      ssdmaxQ("Maximum SSD thickness: ", "0.04"),
      ssdnumQ("Number of SSD thickness steps: ", "10");
    Query fullssdQ(ssdminQ, ssdmaxQ, ssdnumQ);
    fullssdQ.textask();
    ssdthickmin=ssdminQ.val(); ssdthickmax=ssdmaxQ.val(); ssdthickinc=ssdnumQ.val();

    outfile << ssdthickinc << endl;
    ssdthickinc=(ssdthickmax - ssdthickmin)/(ssdthickinc-1);
    if(horflag) {
      cout << "HORIZONTAL AXIS: ";
      nomoreflag=1;  // now we have both axes
    } else {
      cout << "VERTICAL AXIS: ";
      horflag=1;
      ssdthickflag=1;
    }
    cout << "SSD thickness" << endl;
    for(double ssdthick=ssdthickmin; ssdthick<=ssdthickmax+.3*ssdthickinc; ssdthick+=ssdthickinc)
      outfile << ssdthick << " ";
    outfile << endl;
  }

  // Incang has some issues;  incang of gamma or electron?  We'll assume they are approximately the same.
  DblQuery incangQ("Incident angle of gamma ray in degrees", "0.0");
  Query initincangQ(incangQ);
  initincangQ.textask();
  incangmin=incangQ.val();
  incangmax=incangmin;
  if(incangmin<0 && nomoreflag)
    while(incangmin<0) {
      cout << "Already have two axes.  Enter single value for incident angle: " << flush;
      cin >> incangmin;
    }
  if(incangmin<0) {
    DblQuery incangminQ("Minimum incident angle", "0.0"),
      incangmaxQ("Maximum incident angle", "60.0"),
      incangnumQ("Number of incident angle steps", "20");
    Query fullincangQ(incangminQ, incangmaxQ, incangnumQ);
    fullincangQ.textask();
    incangmin=incangminQ.val(); incangmax=incangmaxQ.val(); incanginc=incangnumQ.val();
    outfile << incanginc << endl;
    incanginc=(incangmax - incangmin)/(incanginc - 1);
    if(horflag) {
      cout << "HORIZONTAL AXIS: ";
      nomoreflag=1;  // now we have both axes
    } else {
      cout << "VERTICAL AXIS: ";
      horflag=1;
      incangflag=1;
    }
    cout << "Incident angle" << endl;
    for(double incang=incangmin; incang<=incangmax+.3*incanginc; incang+=incanginc) 
      outfile << incang << " ";
    outfile << endl;
  }

  IntQuery numplanesQ("Number of planes that the electrons pass through", "10");
  Query initnumplanesQ(numplanesQ);
  initnumplanesQ.textask();
  numplanesmin=numplanesQ.val();
  numplanesmax=numplanesmin;
  if(numplanesmin<0 && nomoreflag)
    while(numplanesmin<0) {
      cout << "Already have two axes. Enter single value for number of planes: " << flush;
      cin >> numplanesmin;
    }
  if(numplanesmin<0) {
    IntQuery numplanesminQ("Minimum number of planes", "4"),
      numplanesmaxQ("Maximum number of planes", "20");
    Query fullnumplanesQ(numplanesminQ, numplanesmaxQ);
    fullnumplanesQ.textask();
    numplanesmin=numplanesminQ.val(); numplanesmax=numplanesmaxQ.val();
    numplanesinc=1;
    outfile << (numplanesmax - numplanesmin) + 1 << endl;
    if(horflag) {
      cout << "HORIZONTAL AXIS: ";
      nomoreflag=1;  // now we have both axes
    } else {
      cout << "VERTICAL AXIS: ";
      horflag=1;
    }
     cout << "Number of planes" << endl;


    for(int numplanes=numplanesmin; numplanes<=numplanesmax; numplanes+=numplanesinc)
      outfile << numplanes << " ";
    outfile << endl;
  }

  if(!nomoreflag) // only one axis! tell IDL!
    outfile << 0 << endl;
    
  for (double e=emin; e<=emax*1.1; e*=einc) {
   for(double split=splitmin; split<=splitmax+.3*splitinc; split+=splitinc) {
    for(double pitch=pitchmin; pitch<=pitchmax+.3*pitchinc; pitch+=pitchinc) {
     for(double gap=gapmin; gap<=gapmax+.3*gapinc; gap+=gapinc) {
      for(double conv=convmin; conv<=convmax+.3*convinc; conv+=convinc) {
	for(double ssdthick=ssdthickmin; ssdthick<=ssdthickmax+.3*ssdthickinc; ssdthick+=ssdthickinc){
         for(double incang=incangmin; incang<=incangmax+.3*incanginc; incang+=incanginc) {
	   for(int numplanes=numplanesmin; numplanes<=numplanesmax; numplanes+=numplanesinc) {

  double bob1, bob2;
  bob1=getvar(split*e, pitch, gap, conv, ssdthick, incang/57.29, numplanes);
  bob2=getvar((1-split)*e, pitch, gap, conv, ssdthick, incang/57.29, numplanes);
  psf= sqrt(split*split*bob1 + (1-split)*(1-split)*bob2)*57.29 ;
  outfile << psf << " ";

	   }
	   if(incangflag) outfile << endl;
	 }
	 if(ssdthickflag) outfile << endl;
	}
	if(convflag) outfile << endl;
      }
      if(gapflag) outfile << endl;
     }
     if(pitchflag) outfile << endl;
    }
    if(splitflag) outfile << endl;
   }
   if(eflag) outfile << endl;
  }
  outfile << endl;
  if(horflag)
    cout << "Results written to file 'out.psf'." << endl;
  else
    cout << endl << "1 sigma Gaussian Point-Spread width: " << psf << " degrees." << endl;

}



